@@ -2,7 +2,7 @@
  *
  * Translate algorithm identifiers (or names) to algorithm objects.
  *
- * $Id: algorithms.c,v 1.17 2000/10/06 23:21:29 nisse Exp $ */
+ * $Id: algorithms.c,v 1.18 2000/11/26 17:12:04 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -473,6 +473,15 @@ init_algorithms_options(struct algorithms_options *self,
   self->hostkey_algorithms = NULL;
 }
 
+struct algorithms_options *
+make_algorithms_options(struct alist *algorithms)
+{
+  NEW(algorithms_options, self);
+  init_algorithms_options(self, algorithms);
+
+  return self;
+}
+
 static error_t
 algorithms_argp_parser(int key, char *arg, struct argp_state *state)
 {
