@@ -1,6 +1,6 @@
 /* channel_commands.h
  *
- * $Id: channel_commands.h,v 1.16 2000/11/15 21:45:02 nisse Exp $
+ * $Id: channel_commands.h,v 1.18 2000/11/26 23:25:49 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -32,17 +32,17 @@
 #undef GABA_DECLARE
 
 /* Command to open a new channel. Takes a connection as argument
- * returns a new channel. Raises an exception on error. FIXME: Which
- * exception??? */
+ * returns a new channel. Raises an EXC_CHANNEL_OPEN exception on
+ * error. */
 
 /* GABA:
    (class
      (name channel_open_command)
      (super command)
      (vars
-       ;; This method should return a partially filled in channel,
-       ;; and create a channel open request by calling
-       ;; prepare_channel_open.
+       ; This method should return a partially filled in channel,
+       ; and create a channel open request by calling
+       ; prepare_channel_open.
        (new_channel method "struct ssh_channel *"
                     "struct ssh_connection *connection"
                     "UINT32 local_channel_number"
@@ -63,9 +63,9 @@ void do_channel_open_command(struct command *s,
      (name channel_request_command)
      (super command)
      (vars
-       ;; This method should return a formatted request. The
-       ;; want_reply field in the request should be non_zero iff *c is
-       ;; non-NULL on return.  
+       ; This method should return a formatted request. The
+       ; want_reply field in the request should be non_zero iff *c is
+       ; non-NULL on return.  
        (format_request method "struct lsh_string *"
                        "struct ssh_channel *channel"
 		       "struct command_continuation **c")))
