@@ -1,6 +1,6 @@
 /* exception.h
  *
- * $Id: exception.h,v 1.23 2000/10/31 22:04:10 nisse Exp $ */
+ * $Id: exception.h,v 1.24 2000/11/26 23:25:51 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -199,8 +199,8 @@ make_exception_handler(void (*raise)(struct exception_handler *s,
      (name protocol_exception)
      (super exception)
      (vars
-       ;; A reason code that can be passed in a SSH_MSG_DISCONNECT message.
-       ;; Zero means terminate the connection without sending such a message.
+       ; A reason code that can be passed in a SSH_MSG_DISCONNECT message.
+       ; Zero means terminate the connection without sending such a message.
        (reason . UINT32)))
 */
 
