@@ -1,6 +1,6 @@
 /* handshake.c
  *
- * $Id: handshake.c,v 1.1 2000/11/15 16:43:20 nisse Exp $
+ * $Id: handshake.c,v 1.2 2000/11/26 23:25:52 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -53,7 +53,7 @@
      (super line_handler)
      (vars
        (connection object ssh_connection)
-       ;; Needed for fallback.
+       ; Needed for fallback.
        (fd . int)
        (fallback object ssh1_fallback)))
 */
