@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.118 2000/11/16 22:39:19 nisse Exp $ */
+ * $Id: io.c,v 1.119 2000/11/20 20:47:45 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -1702,94 +1702,6 @@ io_read_file(struct io_backend *backend,
   return make_lsh_fd(backend, fd, e);
 }
 
-#if 0
-static const struct exception *
-check_user_permissions(struct stat *sbuf, const char *fname,
-		       uid_t uid, int secret)
-{
-  mode_t bad = secret ? (S_IRWXG | S_IRWXO) : (S_IWGRP | S_IWOTH);
-
-  if (!S_ISREG(sbuf->st_mode))
-    {
-      werror("io.c: %z is not a regular file.\n",
-	     fname);
-      return make_io_exception(EXC_IO_OPEN_READ, NULL, 0, "Not a regular file");
-    }
-  if (sbuf->st_uid != uid)
-    {
-      werror("io.c: %z not owned by the right user (%i)\n",
-	     fname, uid);
-      return make_io_exception(EXC_IO_OPEN_READ, NULL, 0, "Bad owner");
-    }
-
-  if (sbuf->st_mode & bad)
-    {
-      werror("io.c: Permissions on %z too loose.\n",
-	     fname);
-      return make_io_exception(EXC_IO_OPEN_READ, NULL, 0, "Bad permissions");
-    }
-
-  return NULL;
-}
-
-/* Open a file, but first check that it is owned by the right user and
- * has proper permissions, and change uid before opening the file.
- * Doesn't check permissions on parent directory. Also, doesn't try to
- * forbid symlinks. */
-struct lsh_fd *
-io_read_user_file(struct io_backend *backend,
-		  const char *fname,
-		  uid_t uid, int secret,
-		  const struct exception **x,
-		  struct exception_handler *e)
-{
-  int fd;
-  struct stat sbuf;
-  
-  if (stat(fname, &sbuf) < 0)
-    {
-      if (errno != ENOENT)
-	werror("io_read_user_file: Failed to stat %z (errno = %i): %z\n",
-	       fname, errno, STRERROR(errno));
-
-      *x = make_io_exception(EXC_IO_OPEN_READ, NULL, errno, NULL);
-      return NULL;
-    }
-
-  *x = check_user_permissions(&sbuf, fname, uid, secret);
-
-  if (*x)
-    return NULL;
-
-  fd = open(fname, O_RDONLY);
-  if (fd < 0)
-    {
-      *x = make_io_exception(EXC_IO_OPEN_READ, NULL, errno, NULL);
-      return NULL;
-    }
-
-  if (fstat(fd, &sbuf) < 0)
-    {
-      werror("io_read_user_file: Failed to stat %z (errno = %i): %z\n",
-	     fname, errno, STRERROR(errno));
-      close(fd);
-      
-      *x = make_io_exception(EXC_IO_OPEN_READ, NULL, errno, NULL);
-      return NULL;
-    }
-  
-  *x = check_user_permissions(&sbuf, fname, uid, secret);
-
-  if (*x)
-    {
-      close(fd);
-      return NULL;
-    }
-  
-  return make_lsh_fd(backend, fd, e);
-}
-#endif
-
 void kill_fd(struct lsh_fd *fd)
 {
   fd->super.alive = 0;
