@@ -1,6 +1,6 @@
 /* io_commands.c
  *
- * $Id: io_commands.c,v 1.34 2000/11/14 17:49:06 nisse Exp $
+ * $Id: io_commands.c,v 1.35 2000/11/26 23:25:54 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -265,74 +265,6 @@ STATIC_COLLECT_2_FINAL(collect_listen_callback);
 struct collect_info_1 listen_with_callback =
 STATIC_COLLECT_1(&collect_info_listen_callback_2);
 
-#if 0
-/* A listen function taking three arguments:
- * (listen backend connection port).
- *
- * Suitable for handling forwarding requests. Adds all fd:s to the
- * connection's resource list automatically.
- *
- * NOTE: On second thought, this is not terribly useful, because
- * accepted fd:s should usually be registered on a channels's resource
- * list, not on the connection's. */
-
-/* ;; GABA:
-   (class
-     (name listen_with_connection)
-     (super command)
-     (vars
-       (connection object ssh_connection)
-       (backend object io_backend)))
-*/
-
-static void
-do_listen_with_connection(struct command *s,
-			  struct lsh_object *x,
-			  struct command_continuation *c,
-			  struct exception_handler *e)
-{
-  CAST(listen_with_connection, self, s);
-  CAST(address_info, address, x);
-
-  /* No dns lookups */
-  do_listen(self->backend, address, 0,
-	    self->connection->resources,
-	    NULL,
-	    c, e);
-}
-
-struct command *
-make_listen_with_connection(struct io_backend *backend,
-			    struct ssh_connection *connection)
-{
-  NEW(listen_with_connection, self);
-  self->connection = connection;
-  self->backend = backend;
-
-  self->super.call = do_listen_with_connection;
-
-  return &self->super;
-}
-
-static struct lsh_object *
-collect_listen_connection(struct collect_info_2 *info,
-			  struct lsh_object *a,
-			  struct lsh_object *b)
-{
-  CAST(io_backend, backend, a);
-  CAST(ssh_connection, connection, b);
-  assert(!info->next);
-
-  return &make_listen_with_connection(backend, connection)->super;
-}
-
-static struct collect_info_2 collect_info_listen_connection_2 =
-STATIC_COLLECT_2_FINAL(collect_listen_connection);
-
-struct collect_info_1 listen_with_connection =
-STATIC_COLLECT_1(&collect_info_listen_connection_2);
-#endif
-
 /* GABA:
    (class
      (name connect_continuation)
