@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.124 2000/11/16 22:44:26 nisse Exp $ */
+ * $Id: lsh.c,v 1.128 2000/11/26 23:25:55 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -72,9 +72,6 @@
 #include "lsh_argp.h"
 
 /* Forward declarations */
-struct command_simple options2remote;
-#define OPTIONS2REMOTE (&options2remote.super.super)
-
 struct command_simple lsh_verifier_command;
 #define OPTIONS2VERIFIER (&lsh_verifier_command.super.super)
 
@@ -111,28 +108,15 @@ STATIC_REQUEST_SERVICE(ATOM_SSH_CONNECTION);
 /* GABA:
    (class
      (name lsh_options)
-     (super algorithms_options)
+     (super client_options)
      (vars
-       (backend object io_backend)
+       (algorithms object algorithms_options)
 
        (random object randomness_with_poll)
 
-       (tty object interact)
-       
        (signature_algorithms object alist)
        (home . "const char *")
        
-       ; For i/o exceptions 
-       (handler object exception_handler)
-
-       (exit_code . "int *")
-       
-       (not . int)
-       (port . "char *")
-       (remote object address_info)
-
-       (local_user . "char *")
-       (user . "char *")
        (identity . "char *")
        (with_publickey . int)
 
@@ -154,30 +138,7 @@ STATIC_REQUEST_SERVICE(ATOM_SSH_CONNECTION);
 
        (known_hosts . "const char *")
        
-       ; -1 means default behaviour
-       (with_pty . int)
-
-       (with_remote_peers . int)
-
-       ; Session modifiers
-       (stdin_file . "const char *")
-       (stdout_file . "const char *")
-       (stderr_file . "const char *")
-
-       ; fork() extra processes for handling stdio file-descriptors,
-       ; to avoid setting them in non-blocking mode.
-       (stdin_fork . int)
-       (stdout_fork . int)
-       (stderr_fork . int)
-       
-       ; True if the process's stdin or pty (respectively) has been used. 
-       (used_stdin . int)
-       (used_pty . int)
-       
-       (start_shell . int)
-       (start_gateway . int)
-       (remote_forward . int)
-       (actions struct object_queue)))
+       (start_gateway . int)))
 */
 
 
@@ -187,51 +148,24 @@ make_options(struct io_backend *backend,
 	     int *exit_code)
 {
   NEW(lsh_options, self);
+  init_client_options(&self->super, backend, handler, exit_code);
+
+  self->algorithms
+    = make_algorithms_options(all_symmetric_algorithms());
 
-  init_algorithms_options(&self->super, all_symmetric_algorithms());
-  
-  self->backend = backend;
   self->random = make_default_random(NULL, handler);
-
-  self->tty = make_unix_interact(backend);
   
   self->home = getenv("HOME");
   
   self->signature_algorithms = all_signature_algorithms(&self->random->super);
-  
-  self->handler = handler;
-  self->exit_code = exit_code;
-  
-  self->not = 0;
-  self->remote = NULL;
-  self->local_user = self->user = getenv("LOGNAME");
-
-  /* Default behaviour is to lookup the "ssh" service, and fall back
-   * to port 22 if that fails. */
-  self->port = NULL; 
 
   self->sloppy = 0;
   self->capture = NULL;
   self->capture_file = NULL;
 
   self->known_hosts = NULL;
-  /* self->known_hosts_file = NULL; */
-
-  self->stdin_file = NULL;
-  self->stdout_file = NULL;
-  self->stderr_file = NULL;
-  self->used_stdin = 0;
-
-  self->stdin_fork = 0;
-  self->stdout_fork = 0;
-  self->stderr_fork = 0;
   
-  self->with_pty = -1;
-  self->start_shell = 1;
-  self->with_remote_peers = 0;
   self->start_gateway = 0;
-  
-  object_queue_init(&self->actions);
 
   self->with_publickey = 1;
 
@@ -248,13 +182,6 @@ make_options(struct io_backend *backend,
 }
 
 
-/* Host to connect to */
-DEFINE_COMMAND_SIMPLE(options2remote, a)
-{
-  CAST(lsh_options, options, a);
-  return &options->remote->super;
-}
-
 /* Request ssh-userauth or ssh-connection service, as appropriate,
  * and pass the options as a first argument. */
 DEFINE_COMMAND_SIMPLE(options2service, a)
@@ -285,7 +212,7 @@ do_options2known_hosts(struct command *ignored UNUSED,
       s = tmp->data;
     }
   
-  f = io_read_file(options->backend, s, e);
+  f = io_read_file(options->super.backend, s, e);
 
   if (!f)
     {
@@ -333,7 +260,7 @@ do_options2identities(struct command *ignored UNUSED,
       s = tmp->data;
     }
   
-  f = io_read_file(options->backend, s, e);
+  f = io_read_file(options->super.backend, s, e);
   
   if (!f)
     {
@@ -342,7 +269,8 @@ do_options2identities(struct command *ignored UNUSED,
       COMMAND_RETURN(c, make_object_list(0, -1));
     }
   else
-    COMMAND_CALL(make_spki_read_userkeys(options->signature_algorithms),
+    COMMAND_CALL(make_spki_read_userkeys(options->signature_algorithms,
+					 options->super.tty),
 		 f, c, e);
   
   lsh_string_free(tmp);
@@ -373,9 +301,7 @@ make_options_command(struct lsh_options *options,
   return &self->super;
 }
 
-/* Maps a host key to a (trusted) verifier object.
- *
- * NOTE: Handles only ssh-dss keys. */
+/* Maps a host key to a (trusted) verifier object. */
 
 /* GABA:
    (class
@@ -390,9 +316,8 @@ make_options_command(struct lsh_options *options,
        (sloppy . int)
        ; If non-null, append an ACL for the received key to this file.
        (file object abstract_write)
-       (hash object hash_algorithm) ; For fingerprinting
-       ;; (algorithm object signature_algorithm)
-       ))
+       ; For fingerprinting
+       (hash object hash_algorithm)))
 */
 
 static struct verifier *
@@ -552,8 +477,8 @@ do_lsh_verifier(struct command *s,
   CAST(options_command, self, s);
   CAST_SUBTYPE(spki_context, db, a);
   COMMAND_RETURN(c, make_lsh_host_db(db,
-				     self->options->tty,
-				     self->options->remote,
+				     self->options->super.tty,
+				     self->options->super.remote,
 				     self->options->sloppy,
 				     self->options->capture_file));
 }
@@ -579,17 +504,20 @@ do_lsh_login(struct command *s,
   CAST(options_command, self, s);
   CAST_SUBTYPE(object_list, keys, a);
 
+  struct client_userauth_method *password
+    = make_client_password_auth(self->options->super.tty);
+  
   COMMAND_RETURN(c,
-		 make_client_userauth(ssh_format("%lz", self->options->user),
-				      ATOM_SSH_CONNECTION,
-				      (LIST_LENGTH(keys)
-				       ? make_object_list
-				         (2, 
-					  make_client_publickey_auth(keys),
-					  make_client_password_auth(), 
-					  -1)
-				       : make_object_list(1, make_client_password_auth(), 
-							  -1))));
+		 make_client_userauth
+		 (ssh_format("%lz", self->options->super.user),
+		  ATOM_SSH_CONNECTION,
+		  (LIST_LENGTH(keys)
+		   ? make_object_list
+		   (2, 
+		    make_client_publickey_auth(keys),
+		    password,
+		    -1)
+		   : make_object_list(1, password, -1))));
 }
 
 /* (login options public-keys connection) */
@@ -602,7 +530,7 @@ DEFINE_COMMAND_SIMPLE(lsh_login_command, a)
 			  do_lsh_login)->super;
 }
 
-/* NOTE: options2identities a command_simple, so it must not be
+/* NOTE: options2identities is a command_simple, so it must not be
  * invoked directly. */
 
 /* Requests the ssh-userauth service, log in, and request connection
@@ -629,11 +557,10 @@ DEFINE_COMMAND_SIMPLE(lsh_login_command, a)
      (params
        (connect object command)
        (handshake object handshake_info)
-       (init object make_kexinit)
-       (requests object object_list))
+       (init object make_kexinit))
      (expr (lambda (options)
                ; What to do with the service
-	       ((progn requests)
+	       ((progn (options2actions options))
 	         ; Initialize service
 		 (init_connection_service
 		   ; Either requests ssh-connection service,
@@ -648,101 +575,6 @@ DEFINE_COMMAND_SIMPLE(lsh_login_command, a)
 		       (connect (options2remote options)))))))))
 */
 
-/* GABA:
-   (expr
-     (name make_start_session)
-     (params
-       (open_session object command)
-       (requests object object_list))
-     (expr (lambda (connection)
-       ((progn requests)
-         ; Create a "session" channel
-         (open_session connection)))))
-*/
-
-/* Requests a shell or command, and connects the channel to our stdio. */
-/* GABA:
-   (expr
-     (name lsh_start_session)
-     (params
-       (request object command))
-     (expr
-       (lambda (session)
-          (client_start_io (request session)))))
-*/
-
-static struct command *
-make_lsh_start_session(struct command *request)
-{
-  CAST_SUBTYPE(command, r, lsh_start_session(request));
-  return r;
-}
-
-/* Parse the argument for -R and -L */
-static int
-parse_forward_arg(char *arg,
-		  UINT32 *listen_port,
-		  struct address_info **target)
-{
-  char *first;
-  char *second;
-  char *end;
-  long port;
-  
-  first = strchr(arg, ':');
-  if (!first)
-    return 0;
-
-  second = strchr(first + 1, ':');
-  if (!second || (second == first + 1))
-    return 0;
-
-  if (strchr(second + 1, ':'))
-    return 0;
-
-  port = strtol(arg, &end, 0);
-  if ( (end == arg)  || (end != first)
-       || (port < 0) || (port > 0xffff) )
-    return 0;
-
-  *listen_port = port;
-
-  port = strtol(second + 1, &end, 0);
-  if ( (end == second + 1) || (*end != '\0')
-       || (port < 0) || (port > 0xffff) )
-    return 0;
-
-  *target = make_address_info(ssh_format("%ls", second - first - 1, first + 1), port);
-  
-  return 1;
-}
-
-#if 0
-/* FIXME: Resetting the tty should really be done by the corresponding
- * channel. */
-
-#if WITH_PTY_SUPPORT
-/* Global variable, because we use atexit() rather than on_exit() */
-struct tty_state
-{
-  struct termios mode;
-  int fd;
-} old_tty;
-
-static void reset_tty(void)
-{
-  tty_setattr(old_tty.fd, &old_tty.mode);
-}
-
-static int remember_tty(int fd)
-{
-  old_tty.fd = fd;
-  return (tty_getattr(fd, &old_tty.mode)
-	  && !atexit(reset_tty));
-}
-#endif /* WITH_PTY_SUPPORT */
-#endif
-
 
 /* Option parsing */
 
@@ -775,8 +607,6 @@ static const struct argp_option
 main_options[] =
 {
   /* Name, key, arg-name, flags, doc, group */
-  { "port", 'p', "Port", 0, "Connect to this port.", 0 },
-  { "user", 'l', "User name", 0, "Login as this user.", 0 },
   { "identity", 'i',  "Identity key", 0, "Use this key to authenticate.", 0 },
   { "publickey", OPT_PUBLICKEY, NULL, 0,
     "Try publickey user authentication (default).", 0 },
@@ -804,37 +634,11 @@ main_options[] =
     "Request the ssh-userauth service (default, unless SRP is being used).", 0 },
   { "no-userauth", OPT_USERAUTH | ARG_NOT, NULL, 0,
     "Request the ssh-userauth service (default if SRP is used).", 0 },
-  
-  { NULL, 0, NULL, 0, "Actions:", 0 },
-  { "forward-local-port", 'L', "local-port:target-host:target-port", 0, "", 0 },
-  { "forward-remote-port", 'R', "remote-port:target-host:target-port", 0, "", 0 },
-  { "nop", 'N', NULL, 0, "No operation (suppresses the default action, "
-    "which is to spawn a remote shell)", 0 },
-  { "execute", 'E', "command", 0, "Execute a command on the remote machine", 0 },
-  { "shell", 'S', "command", 0, "Spawn a remote shell", 0 },
-  { "gateway", 'G', NULL, 0, "Setup a local gateway", 0 },
-  { NULL, 0, NULL, 0, "Modifiers that apply to port forwarding:", 0 },
-  { "remote-peers", 'g', NULL, 0, "Allow remote access to forwarded ports", 0 },
-  { "no-remote-peers", 'g' | ARG_NOT, NULL, 0, 
-    "Disallow remote access to forwarded ports (default).", 0 },
 
-  { NULL, 0, NULL, 0, "Modifiers that apply to remote execution:", 0 },
-  { "stdin", OPT_STDIN, "Filename", 0, "Redirect stdin", 0},
-  { "no-stdin", OPT_STDIN | ARG_NOT, NULL, 0, "Redirect stdin from /dev/null", 0}, 
-  { "stdout", OPT_STDOUT, "Filename", 0, "Redirect stdout", 0},
-  { "no-stdout", OPT_STDOUT | ARG_NOT, NULL, 0, "Redirect stdout to /dev/null", 0}, 
-  { "stderr", OPT_STDERR, "Filename", 0, "Redirect stderr", 0},
-  { "no-stderr", OPT_STDERR | ARG_NOT, NULL, 0, "Redirect stderr to /dev/null", 0}, 
-  { "cvs-workaround", OPT_FORK_STDIO, "i?o?e?", OPTION_ARG_OPTIONAL,
-    "fork extra processes to read one or more of the stdio file "
-    "descriptors, to avoid setting them in non-blocking mode.", 0 },
-  
-#if WITH_PTY_SUPPORT
-  { "pty", 't', NULL, 0, "Request a remote pty (default).", 0 },
-  { "no-pty", 't' | ARG_NOT, NULL, 0, "Don't request a remote pty.", 0 },
-#endif /* WITH_PTY_SUPPORT */
-  { NULL, 0, NULL, 0, "Universal not:", 0 },
-  { "no", 'n', NULL, 0, "Inverts the effect of the next modifier", 0 },
+  /* ACtions */
+  { "forward-remote-port", 'R', "remote-port:target-host:target-port",
+    0, "", CLIENT_ARGP_ACTION_GROUP },
+  { "gateway", 'G', NULL, 0, "Setup a local gateway", 0 },
   { NULL, 0, NULL, 0, NULL, 0 }
 };
 
@@ -842,277 +646,17 @@ main_options[] =
 static const struct argp_child
 main_argp_children[] =
 {
+  { &client_argp, 0, "", 0 },
   { &algorithms_argp, 0, "", 0 },
   { &werror_argp, 0, "", 0 },
   { NULL, 0, NULL, 0}
 };
 
-/* FIXME: Moves to client.c */
-static int
-fork_input(int in)
-{
-  /* pipe[0] for reading, pipe[1] for writing. */
-  int pipe[2];
-
-  if (!lsh_make_pipe(pipe))
-    return -1;
-
-  switch (fork())
-    {
-    case -1:
-      /* Error */
-      return -1;
-    case 0:
-      close(pipe[0]);
-      if (lsh_copy_file(in, pipe[1]))
-	_exit(EXIT_SUCCESS);
-      else
-	_exit(EXIT_FAILURE);
-    default:
-      /* Parent */
-      close(pipe[1]);
-      return pipe[0];
-    }
-}
-
-static int
-fork_output(int out)
-{
-  /* pipe[0] for reading, pipe[1] for writing. */
-  int pipe[2];
-
-  if (!lsh_make_pipe(pipe))
-    return -1;
-
-  switch (fork())
-    {
-    case -1:
-      /* Error */
-      return -1;
-    case 0:
-      close(pipe[1]);
-      if (lsh_copy_file(pipe[0], out))
-	_exit(EXIT_SUCCESS);
-      else
-	_exit(EXIT_FAILURE);
-    default:
-      /* Parent */
-      close(pipe[0]);
-      return pipe[1];
-    }
-}
-
-/* Create a session object. stdout and stderr are shared (although
- * with independent lsh_fd objects). stdin can be used by only one
- * session (until something "session-control"/"job-control" is added).
- * */
-static struct ssh_channel *
-make_lsh_session(struct lsh_options *self)
-{
-  int in;
-  int out;
-  int err;
-
-  debug("lsh.c: Setting up stdin\n");
-
-  if (self->stdin_file)
-    in = open(self->stdin_file, O_RDONLY);
-  else
-    {
-      if (self->used_stdin)
-	in = open("/dev/null", O_RDONLY);
-      else 
-	{
-	  in = (self->stdin_fork ? fork_input : dup)(STDIN_FILENO);
-	  self->used_stdin = 1;
-	}
-    }
-    
-  if (in < 0)
-    {
-      werror("lsh: Can't dup/open stdin (errno = %i): %z!\n",
-	     errno, strerror(errno));
-      return NULL;
-    }
-
-  debug("lsh.c: Setting up stdout\n");
-
-  if (self->stdout_file)
-    out = open(self->stdout_file, O_WRONLY | O_CREAT, 0666);
-  else if (self->stdout_fork)
-    out = fork_output(STDOUT_FILENO);
-  else
-    out = dup(STDOUT_FILENO);
-
-  if (out < 0)
-    {
-      werror("lsh: Can't dup/open stdout (errno = %i): %z!\n",
-	     errno, strerror(errno));
-      close(in);
-      return NULL;
-    }
-
-  debug("lsh.c: Setting up stderr\n");
-  
-  if (self->stderr_file)
-    err = open(self->stderr_file, O_WRONLY | O_CREAT, 0666);
-  else if (self->stderr_fork)
-    err = fork_output(STDERR_FILENO);
-  else
-    {
-      err = dup(STDERR_FILENO);
-      set_error_stream(STDERR_FILENO, 1);
-    }
-
-  if (err < 0) 
-    {
-      werror("lsh: Can't dup/open stderr!\n");
-      close(in);
-      close(out);
-      return NULL;
-    }
-
-  /* Clear options */
-  self->stdin_file = self->stdout_file = self->stderr_file = NULL;
-  
-  return make_client_session
-    (io_read(make_lsh_fd(self->backend, in, self->handler),
-	     NULL, NULL),
-     io_write(make_lsh_fd(self->backend, out, self->handler),
-	      BLOCK_SIZE, NULL),
-     io_write(make_lsh_fd(self->backend, err, self->handler),
-	      BLOCK_SIZE, NULL),
-     WINDOW_SIZE,
-     self->exit_code);
-}
-
-/* FIXME: Moves to client.c */
-/* Create an interactive session */
-static struct command *
-lsh_shell_session(struct lsh_options *self)
-{
-  struct command *get_pty = NULL;
-  struct command *get_shell;
-  
-  struct object_list *session_requests;
-  struct ssh_channel *session = make_lsh_session(self);
-
-  if (!session)
-    return NULL;
-  
-#if WITH_PTY_SUPPORT
-  if (self->with_pty && !self->used_pty)
-    {
-      self->used_pty = 1;
-      
-      if (self->tty && INTERACT_IS_TTY(self->tty))
-	{
-	  get_pty = make_pty_request(self->tty);
-	  if (!get_pty)
-	    {
-	      werror("lsh: Can't use tty (probably getattr or atexit() failed.\n");
-	    }
-	}
-      else
-	{
-	  werror("lsh: No tty available.\n");
-	}
-    }
-
-  get_shell = make_lsh_start_session(&request_shell.super);
-  
-  /* FIXME: We need a non-varargs constructor for lists. */
-  if (get_pty)
-    session_requests
-      = make_object_list(2,
-			 /* Ignore EXC_CHANNEL_REQUEST for the pty allocation call. */
-			 make_catch_apply
-			 (make_catch_handler_info(EXC_ALL, EXC_CHANNEL_REQUEST,
-						  0, NULL),
-			  get_pty),
-			 get_shell, -1);
-  else
-#endif /* WITH_PTY_SUPPORT */
-    session_requests = make_object_list(1, get_shell, -1);
-
-  {
-    CAST_SUBTYPE(command, r,
-		 make_start_session
-		 (make_open_session_command(session), session_requests));
-    return r;
-  }
-}
-
-/* Create a session executing a command line */
-static struct command *
-lsh_command_session(struct lsh_options *self,
-		    struct lsh_string *command)
-{
-  struct ssh_channel *session = make_lsh_session(self);
-
-  if (session)
-    {
-      CAST_SUBTYPE(command, r,
-		   make_start_session
-		   (make_open_session_command(session),
-		    make_object_list
-		    (1, make_lsh_start_session(make_exec_request(command)),
-		     -1)));
-      return r;
-    }
-  
-  return NULL;
-}
-
-static struct command *
-lsh_add_action(struct lsh_options *self,
-	       struct command *action)
-{
-  if (action)
-    object_queue_add_tail(&self->actions, &action->super);
-
-  return action;
-}
-
-/* NOTE: Some of the original quoting is lost here. */
-static struct lsh_string *
-rebuild_command_line(unsigned argc, char **argv)
-{
-  unsigned length;
-  unsigned i;
-  unsigned pos;
-  struct lsh_string *r;
-  unsigned *alengths = alloca(sizeof(unsigned) * argc);
-  
-  assert (argc);
-  length = argc - 1; /* Number of separating spaces. */
-
-  for (i = 0; i<argc; i++)
-    {
-      alengths[i] = strlen(argv[i]);
-      length += alengths[i];
-    }
-
-  r = lsh_string_alloc(length);
-  memcpy(r->data, argv[0], alengths[0]);
-  pos = alengths[0];
-  for (i = 1; i<argc; i++)
-    {
-      r->data[pos++] = ' ';
-      memcpy(r->data + pos, argv[i], alengths[i]);
-      pos += alengths[i];
-    }
-
-  assert(pos == r->length);
-
-  return r;
-}
-
 #define CASE_ARG(opt, attr, none)		\
   case opt:					\
-    if (self->not)				\
+    if (self->super.not)			\
       {						\
-        self->not = 0;				\
+        self->super.not = 0;			\
 						\
       case opt | ARG_NOT:			\
         self->attr = none;			\
@@ -1124,9 +668,9 @@ rebuild_command_line(unsigned argc, char **argv)
 
 #define CASE_FLAG(opt, flag)			\
   case opt:					\
-    if (self->not)				\
+    if (self->super.not)			\
       {						\
-        self->not = 0;				\
+        self->super.not = 0;			\
 						\
       case opt | ARG_NOT:			\
         self->flag = 0;				\
@@ -1147,50 +691,17 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       return ARGP_ERR_UNKNOWN;
     case ARGP_KEY_INIT:
       state->child_inputs[0] = &self->super;
-      state->child_inputs[1] = NULL;
+      state->child_inputs[1] = self->algorithms;
+      state->child_inputs[2] = NULL;
       break;
-    case ARGP_KEY_NO_ARGS:
-      argp_usage(state);
-      break;
-    case ARGP_KEY_ARG:
-      if (!state->arg_num)
-	{
-	  if (self->port)
-	    self->remote = make_address_info_c(arg, self->port, 0);
-	  else
-	    self->remote = make_address_info_c(arg, "ssh", 22);
-	  
-	  if (!self->remote)
-	    argp_error(state, "Invalid port or service '%s'.", self->port);
-
-	  break;
-	}
-      else
-	/* Let the next case parse it.  */
-	return ARGP_ERR_UNKNOWN;
-
-      break;
-    case ARGP_KEY_ARGS:
-      lsh_add_action(self,
-		     lsh_command_session(self,
-					 rebuild_command_line(state->argc - state->next,
-							      state->argv + state->next)));
-      self->start_shell = 0;
-      break;
-
+      
     case ARGP_KEY_END:
       if (!self->home)
 	{
 	  argp_error(state, "No home directory. Please set HOME in the environment.");
 	  break;
 	}
-	  
-      if (!self->user)
-	{
-	  argp_error(state, "No user name given. Use the -l option, or set LOGNAME in the environment.");
-	  break;
-	}
-	    
+
       if (self->with_dh_keyexchange < 0)
 	self->with_dh_keyexchange = !self->with_srp_keyexchange;
       
@@ -1204,18 +715,21 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	  if (self->with_srp_keyexchange)
 	    {
 	      LIST(self->kex_algorithms)[i++] = ATOM_SRP_RING1_SHA1_LOCAL;
-	      ALIST_SET(self->super.algorithms,
+	      ALIST_SET(self->algorithms->algorithms,
 			ATOM_SRP_RING1_SHA1_LOCAL,
-			make_srp_client(make_srp1(&self->random->super),
-					ssh_format("%lz", self->user)));
+			&make_srp_client(make_srp1(&self->random->super),
+					 self->super.tty,
+					 ssh_format("%lz", self->super.user))
+			->super);
 	    }
 #endif /* WITH_SRP */
 	  if (self->with_dh_keyexchange)
 	    {
 	      LIST(self->kex_algorithms)[i++] = ATOM_DIFFIE_HELLMAN_GROUP1_SHA1;
-	      ALIST_SET(self->super.algorithms,
+	      ALIST_SET(self->algorithms->algorithms,
 			ATOM_DIFFIE_HELLMAN_GROUP1_SHA1,
-			make_dh_client(make_dh1(&self->random->super)));
+			&make_dh_client(make_dh1(&self->random->super))
+			->super);
 	    }
 	}
       else
@@ -1281,7 +795,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	if (s)
 	  {
 	    struct lsh_fd *f
-	      = io_write_file(self->backend, s,
+	      = io_write_file(self->super.backend, s,
 			      O_CREAT | O_APPEND | O_WRONLY,
 			      0600, 500, NULL,
 			      make_report_exception_handler
@@ -1299,29 +813,19 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	  }
 	lsh_string_free(tmp);
       }
-	
-#if WITH_TCP_FORWARD
-      if (self->remote_forward)
-	lsh_add_action(self,
-		       make_install_fix_channel_open_handler
-		       (ATOM_FORWARDED_TCPIP, &channel_open_forwarded_tcpip));
-#endif /* WITH_TCP_FORWARD */
-      
-      /* Add shell action */
-      if (self->start_shell)
-	lsh_add_action(self, lsh_shell_session(self));
 
       if (self->start_gateway)
 	{
 	  struct local_info *gateway;
-	  if (!self->local_user)
+	  if (!self->super.local_user)
 	    {
 	      argp_error(state, "You have to set LOGNAME in the environment, "
 			 " if you want to use the gateway feature.");
 	      break;
 	    }
-	  gateway = make_gateway_address(self->local_user, self->user,
-					 self->remote);
+	  gateway = make_gateway_address(self->super.local_user,
+					 self->super.user,
+					 self->super.remote);
 
 	  if (!gateway)
 	    {
@@ -1330,35 +834,21 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	      break;
 	    }
 	      
-	  lsh_add_action(self,
-			 make_gateway_setup
-			 (make_listen_local(self->backend, gateway)));
-	}
-	
-      if (object_queue_is_empty(&self->actions))
-	{
-	  argp_error(state, "No actions given.");
-	  break;
+	  client_add_action(&self->super,
+			    make_gateway_setup
+			    (make_listen_local(self->super.backend, gateway)));
 	}
 
       /* Start background poll */
       RANDOM_POLL_BACKGROUND(self->random->poller);
       
       break;
-
-    case 'p':
-      self->port = arg;
-      break;
-
-    case 'l':
-      self->user = arg;
-      break;
-
+      
     case 'i':
       self->identity = optarg;
       break;
 
-      CASE_FLAG(OPT_PUBLICKEY, with_publickey);
+    CASE_FLAG(OPT_PUBLICKEY, with_publickey);
 
     case OPT_HOST_DB:
       self->known_hosts = optarg;
@@ -1376,104 +866,54 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       self->capture = arg;
       break;
 
-      CASE_FLAG(OPT_DH, with_dh_keyexchange);
-      CASE_FLAG(OPT_SRP, with_srp_keyexchange);
+    CASE_FLAG(OPT_DH, with_dh_keyexchange);
+    CASE_FLAG(OPT_SRP, with_srp_keyexchange);
 
-      CASE_FLAG(OPT_USERAUTH, with_userauth);
+    CASE_FLAG(OPT_USERAUTH, with_userauth);
 
-    case 'E':
-      lsh_add_action(self, lsh_command_session(self, ssh_format("%lz", arg)));
-      break;
-
-    case 'S':
-      lsh_add_action(self, lsh_shell_session(self));
-      break;
-      
+#if 0
     case 'L':
       {
 	UINT32 listen_port;
 	struct address_info *target;
 
-	if (!parse_forward_arg(arg, &listen_port, &target))
+	if (!client_parse_forward_arg(arg, &listen_port, &target))
 	  argp_error(state, "Invalid forward specification '%s'.", arg);
 
-	lsh_add_action(self, make_forward_local_port
-		       (self->backend,
-			make_address_info((self->with_remote_peers
-					   ? NULL
-					   : ssh_format("%lz", "127.0.0.1")),
-					  listen_port),
-			target));
+	client_add_action(&self->super, make_forward_local_port
+			  (self->super.backend,
+			   make_address_info((self->super.with_remote_peers
+					      ? NULL
+					      : ssh_format("%lz", "127.0.0.1")),
+					     listen_port),
+			   target));
 	break;
       }      
-
+#endif
+      
     case 'R':
       {
 	UINT32 listen_port;
 	struct address_info *target;
 
-	if (!parse_forward_arg(arg, &listen_port, &target))
+	if (!client_parse_forward_arg(arg, &listen_port, &target))
 	  argp_error(state, "Invalid forward specification '%s'.", arg);
 
-	lsh_add_action(self, make_forward_remote_port
-		       (self->backend,
-			make_address_info((self->with_remote_peers
-					   ? ssh_format("%lz", "0.0.0.0")
-					   : ssh_format("%lz", "127.0.0.1")),
-					  listen_port),
-			target));
-	self->remote_forward = 1;
+	client_add_action(&self->super, make_forward_remote_port
+			  (self->super.backend,
+			   make_address_info((self->super.with_remote_peers
+					      ? ssh_format("%lz", "0.0.0.0")
+					      : ssh_format("%lz", "127.0.0.1")),
+					     listen_port),
+			   target));
+
+	self->super.remote_forward = 1;
 	break;
       }      
       
-    case 'N':
-      self->start_shell = 0;
-      break;
-
-      CASE_FLAG('G', start_gateway);
-      CASE_FLAG('g', with_remote_peers);
-
-#if WITH_PTY_SUPPORT
-      CASE_FLAG('t', with_pty);
-#endif /* WITH_PTY_SUPPORT */
-
-      CASE_ARG(OPT_STDIN, stdin_file, "/dev/null");
-      CASE_ARG(OPT_STDOUT, stdout_file, "/dev/null"); 
-      CASE_ARG(OPT_STDERR, stderr_file, "/dev/null");
-
-    case OPT_FORK_STDIO:
-      if (!arg)
-	self->stdin_fork = self->stdout_fork = self->stderr_fork = 1;
-      else
-	{
-	  int i;
-	  for (i = 0; arg[i]; i++)
-	    switch(arg[i])
-	      {
-	      case 'i': case 'I':
-		self->stdin_fork = 1;
-		break;
-	      case 'o': case 'O':
-		self->stdout_fork = 1;
-		break;
-	      case 'e': case 'E':
-		self->stderr_fork = 1;
-		break;
-	      default:
-		argp_error(state, "The argument to --cvs-workaround should "
-			   "be one or more of the characters 'i' (stdin), "
-			   "'o' (stdout) and 'e' (stderr).");
-		goto loop_done;
-	      }
-	loop_done:
-	}
-      break;
-		
-    case 'n':
-      self->not = !self->not;
-      break;
+    CASE_FLAG('G', start_gateway);
     }
-  
+
   return 0;
 }
 
@@ -1580,16 +1020,15 @@ int main(int argc, char **argv)
 			    "lsh - a free ssh", NULL,
 			    SSH_MAX_PACKET,
 			    &options->random->super,
-			    options->super.algorithms,
+			    options->algorithms->algorithms,
 			    NULL),
 	make_simple_kexinit(&options->random->super,
 			    options->kex_algorithms,
-			    options->super.hostkey_algorithms,
-			    options->super.crypto_algorithms,
-			    options->super.mac_algorithms,
-			    options->super.compression_algorithms,
-			    make_int_list(0, -1)),
-	queue_to_list(&options->actions));
+			    options->algorithms->hostkey_algorithms,
+			    options->algorithms->crypto_algorithms,
+			    options->algorithms->mac_algorithms,
+			    options->algorithms->compression_algorithms,
+			    make_int_list(0, -1)));
     
     CAST_SUBTYPE(command, lsh_connect, o);
 
