@@ -2,7 +2,7 @@
  *
  * System dependant password related functions.
  *
- * $Id: server_password.c,v 1.30 2000/04/21 22:29:41 nisse Exp $
+ * $Id: server_password.c,v 1.31 2000/10/15 20:22:26 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -102,20 +102,7 @@ do_authenticate(struct userauth *s,
 	      return;
 	    }
 
-	  if (USER_VERIFY_PASSWORD(user, password, 1))
-	    {
-	      COMMAND_RETURN(c, user);
-	      return;
-	    }
-	  else
-	    {
-	      static const struct exception bad_passwd
-		= STATIC_EXCEPTION(EXC_USERAUTH, "Wrong password");
-
-	      KILL(user);
-	      EXCEPTION_RAISE(e, &bad_passwd);
-	      return;
-	    }
+	  USER_VERIFY_PASSWORD(user, password, c, e);
 	}
     }
   
