@@ -139,7 +139,7 @@ do_userauth_continuation(struct command_continuation *s,
   
   /* Ignore any further userauth messages. */
   for (i = SSH_FIRST_USERAUTH_GENERIC; i < SSH_FIRST_CONNECTION_GENERIC; i++) 
-    self->connection->dispatch[i] = self->connection->ignore;
+    self->connection->dispatch[i] = &connection_ignore_handler;
 
   COMMAND_RETURN(self->up, self->connection);
 
