@@ -2,7 +2,7 @@
  *
  * User-related functions on UN*X
  *
- * $Id: unix_user.c,v 1.28 2000/11/14 18:09:49 nisse Exp $
+ * $Id: unix_user.c,v 1.30 2000/11/16 22:46:57 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -214,13 +214,11 @@ static struct resource *
 make_process_resource(pid_t pid, int signal)
 {
   NEW(process_resource, self);
-  self->super.alive = 1;
+  resource_init(&self->super, do_kill_process);
 
   self->pid = pid;
   self->signal = signal;
 
-  self->super.kill = do_kill_process;
-
   return &self->super;
 }
 
@@ -603,10 +601,7 @@ do_read_file(struct lsh_user *u,
     case 0:
       /* Child */
       {
-#define BUF_SIZE 1024
 	int fd;
-	char buf[BUF_SIZE];
-
 	close(out[0]);
 	
 	if ( (me != user->super.uid) && (setuid(user->super.uid) < 0) )
@@ -636,47 +631,11 @@ do_read_file(struct lsh_user *u,
 	    werror("unix_user.c: do_read_file(): %z\n", x->msg);
 	    _exit(EXIT_FAILURE);
 	  }
-	
-	/* Copying loop */
-	for (;;)
-	  {
-	    int res = read(fd, buf, BUF_SIZE);
-	    switch (res)
-	      {
-	      case -1:
-		if (errno != EINTR)
-		  {
-		    werror("unix_user.c: do_read_file(): read failed (errno = %i): %z\n",
-			   errno, STRERROR(errno));
-		    _exit(EXIT_FAILURE);
-		  }
-		break;
-	      case 0:
-		/* EOF */
-		_exit(EXIT_SUCCESS);
-	      default:
-		{
-		  unsigned length = res;
-		  unsigned i;
-		  for (i = 0; i<length; )
-		    {
-		      res = write(out[1], buf + i, length - i);
-		      if (res < 0)
-			{
-			  if (errno != EINTR)
-			    {
-			      werror("unix_user.c: do_read_file(): write failed (errno = %i): %z\n",
-				     errno, STRERROR(errno));
-			      _exit(EXIT_FAILURE);
-			    }
-			}
-		      else
-			i += res;
-		    }
-		}
-	      }
-	  }
-#undef BUF_SIZE
+
+	if (lsh_copy_file(fd, out[1]))
+	  _exit(EXIT_SUCCESS);
+	else
+	  _exit(EXIT_FAILURE);
       }
     }
 }
