@@ -2,7 +2,7 @@
  *
  * zlib compression algorithm
  * 
- * $Id: zlib.c,v 1.28 2000/04/20 16:18:23 nisse Exp $ */
+ * $Id: zlib.c,v 1.29 2000/11/26 23:26:04 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -49,7 +49,7 @@ static void do_free_zstream(z_stream *z);
      (name zlib_instance)
      (super compress_instance)
      (vars
-       ;; Fail before producing larger packets than this
+       ; Fail before producing larger packets than this
        (max . UINT32)
        (rate . UINT32)
        (f pointer (function int "z_stream *" int))
